/*
 ============================================================================
 Name        : IOPerf.c
 Author      : utente
 Version     :
 Copyright   : Your copyright notice
 Description : Valutazione dei tempi per la creazione e l'accesso a file
				 in funzione del numero di file presenti nella directory
 ============================================================================
 */

#include <ourhdr.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <sys/timeb.h>
#include <sys/time.h>
#include <sys/resource.h>


#define MaxFiles 100000

double TimeDiff( struct timeb Start, struct timeb Stop);

double TimeDiff2( struct timeval Start, struct timeval Stop);


int main(int argc, char * argv[]) {
	int fd, i, numfiles, numcicli;
	char fname[100];
	char a[MaxFiles][12];
	char error_msg[250];
	char *b;

	struct timeb TStart, TStop;
	struct rusage RStart, RStop;


	if (argc < 3) {
		fprintf(stderr, "Errore negli argomenti.\nUsage: %s #numfiles #numcicli\n", argv[0]);
		exit(-1);
	}
	numfiles = atoi(argv[1]);
	numcicli = atoi(argv[2]);
	if (numfiles > MaxFiles) {
		fprintf(stderr, "Errore negli argomenti.\nnumfiles (%d) must be less than %d #numcicli\n", numfiles, MaxFiles);
		exit(-1);
	}

	// inizio creazione files
	printf("Inizio creazione %d files\n", numfiles);
	srandom(time(NULL)); // inizializza il generatore di numeri casuali, per non creare sempre la stessa sequenza di file

	ftime( &TStart);
	getrusage( RUSAGE_SELF, &RStart);
	for(i = 0; i < numfiles ; i++)
	{
		sprintf(fname, "%010ld", random());

		if ((fd = open(fname, O_WRONLY | O_CREAT | O_TRUNC, 0644 )) < 0)
		{
			sprintf(error_msg,"Opening Output file n.%d: %s", i, fname);
			err_sys(error_msg);
		}
		// salva una copia del file name appena creato per il secondo loop
		strcpy(a[i], fname);
		putchar('.');	// stampa un punto per ogni file
		close(fd);		// occorre chiudere il file altrimenti appena raggiunge MAXOPENFILES la open fallisce
	}
	ftime( &TStop);
	getrusage( RUSAGE_SELF, &RStop);
	printf("\nFine creazione file\n");

	printf("\nTotal Elapsed Time: %.3f\n", TimeDiff(TStart, TStop));
	printf("Total Sys Time    : %.3f\n", TimeDiff2(RStart.ru_stime, RStop.ru_stime));
	printf("Total User Time   : %.3f\n", TimeDiff2(RStart.ru_utime, RStop.ru_utime));
	printf("Tempo x create    : %.3f msec\n", (TimeDiff(TStart, TStop) * 1000.) / numfiles);
	printf("Blocking I/O oper.: %ld (in) %ld (out)\n",
				RStop.ru_inblock-RStart.ru_inblock, RStop.ru_oublock-RStart.ru_oublock);

	printf("Inizio fase 2\n");
	// seconda fase, apre numcicli file per verificare il tempo di accesso medio
	ftime( &TStart);
	getrusage( RUSAGE_SELF, &RStart);

	for(i = 0; i < numcicli; i++)
	{
		// sceglie un file a caso tra quelli creati lo salva in b per poi aprirlo
		b = a[random() % numfiles];
		// printf("Opening: <%s>\n", b);
		putchar('.');
		if ((fd = open( b, O_RDWR)) < 0)
		{
			sprintf(error_msg,"Opening Input file n.%d: %s", i, b);
			err_sys(error_msg);
		}
		close(fd);
	}
	ftime( &TStop);
	getrusage( RUSAGE_SELF, &RStop);

	printf("\nTotal Elapsed Time: %.3f\n", TimeDiff(TStart, TStop));
	printf("Total Sys Time    : %.3f\n", TimeDiff2(RStart.ru_stime, RStop.ru_stime));
	printf("Total User Time   : %.3f\n", TimeDiff2(RStart.ru_utime, RStop.ru_utime));
	printf("Tempo x open      : %.3f msec\n", (TimeDiff(TStart, TStop) * 1000.) / numcicli);
	printf("Blocking I/O oper.: %ld (in) %ld (out)\n",
				RStop.ru_inblock-RStart.ru_inblock, RStop.ru_oublock-RStart.ru_oublock);

	return EXIT_SUCCESS;
}

double TimeDiff( struct timeb Start, struct timeb Stop)
{
	double diff;

	if (Stop.millitm < Start.millitm) {
		Stop.millitm += 1000;
		Stop.time -= 1;
	}
	diff = (double) (Stop.time - Start.time) + (Stop.millitm - Start.millitm) / 1000.;
	return diff;
}

double TimeDiff2( struct timeval Start, struct timeval Stop)
{
	if (Stop.tv_usec < Start.tv_usec) {
		Stop.tv_usec += 1000;
		Stop.tv_sec -= 1;
	}
	return (double) (Stop.tv_sec - Start.tv_sec + (Stop.tv_usec - Start.tv_usec) / 1000000.);
}


